'use strict';

var _must = require('must');

var _must2 = _interopRequireDefault(_must);

var _eventToPromise = require('event-to-promise');

var _eventToPromise2 = _interopRequireDefault(_eventToPromise);

var _jsonRpcPeer = require('json-rpc-peer');

var _jsonRpcPeer2 = _interopRequireDefault(_jsonRpcPeer);

var _ws = require('ws');

var _index = require('./index');

var _index2 = _interopRequireDefault(_index);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// ===================================================================

// ===================================================================

function noop() {}

// ===================================================================

/* eslint-env mocha */

describe('Client', function () {
  var serverPort = void 0;
  before(function (done) {
    new _ws.Server({
      host: 'localhost',
      port: 0
    }, function () {
      serverPort = this.address().port;

      done();
    }).on('connection', function (socket) {
      var jsonRpc = new _jsonRpcPeer2.default(function (message) {
        if (message.type === 'notification') {
          return;
        }

        if (message.method === 'identity') {
          return message.params[0];
        }

        if (message.method === 'error') {
          throw new _jsonRpcPeer.JsonRpcError(message.params[0]);
        }
      });

      jsonRpc.on('data', function (data) {
        if (socket.readyState === socket.OPEN) {
          socket.send(data);
        }
      });
      socket.on('message', function (message) {
        jsonRpc.write(message);
      });
    });
  });

  var client = void 0;
  beforeEach(function () {
    client = new _index2.default('ws://localhost:' + serverPort);
  });
  afterEach(function () {
    client.close().catch(noop);
  });

  it('emits a `connected` event on connection', function () {
    client.connect();

    return (0, _eventToPromise2.default)(client, 'connected');
  });

  it('emits a `disconnected` event on disconnection', function () {
    return client.connect().then(function () {
      client.close();

      return (0, _eventToPromise2.default)(client, 'disconnected');
    });
  });

  describe('#connect()', function () {
    it('returns a promise which rejects if connecting', function () {
      client.connect();
      return (0, _must2.default)(client.connect()).to.reject();
    });

    it('returns a promise which rejects if connected', function () {
      return client.connect().then(function () {
        return (0, _must2.default)(client.connect()).to.reject();
      });
    });

    it('returns a promise which resolves when connected', function () {
      return client.connect();
    });

    it('returns a promise which rejects on connection error', function () {
      client = new _index2.default('ws://localhost:-1');

      return (0, _must2.default)(client.connect()).to.reject();
    });
  });

  describe('#close()', function () {
    it('returns a promise which rejects if disconnected', function () {
      return (0, _must2.default)(client.close()).to.reject();
    });

    it('returns a promise which resolve when disconnected', function () {
      return client.connect().then(function () {
        return client.close();
      });
    });

    it('aborts the connection if connecting', function () {
      var promise = client.connect();

      return client.close().then(function () {
        return (0, _must2.default)(promise).to.reject();
      });
    });

    it('fails all waiting calls', function () {
      return client.connect().then(function () {
        var promise = client.call('foo');
        return client.close().then(function () {
          return (0, _must2.default)(promise).to.reject.to.an.error(_index.ConnectionError);
        });
      });
    });
  });

  describe('#call()', function () {
    it('returns a promise which rejects if not connected', function () {
      return (0, _must2.default)(client.call('foo')).to.reject();
    });

    it('returns a promise which resolves with the result of the request', function () {
      return client.connect().then(function () {
        return client.call('identity', [42]);
      }).then(function (result) {
        (0, _must2.default)(result).to.equal(42);
      });
    });

    it('returns a promise which rejects with the error of the request', function () {
      return client.connect().then(function () {
        return (0, _must2.default)(client.call('error', ['an error'])).to.reject.to.an.error('an error');
      });
    });
  });

  describe('#status', function () {
    it('is `disconnected` when created', function () {
      (0, _must2.default)(client.status).to.equal('disconnected');
    });

    it('is `connecting` during connection', function () {
      client.connect();

      (0, _must2.default)(client.status).to.equal('connecting');
    });

    it('is `connected` after connection', function () {
      return client.connect().then(function () {
        (0, _must2.default)(client.status).to.equal('connected');
      });
    });

    it('is `diconnected` after disconnection', function () {
      return client.connect().then(function () {
        client.close();

        (0, _must2.default)(client.status).to.equal('disconnected');
      });
    });
  });
});
//# sourceMappingURL=index.spec.js.map